# Obtained and modified from:
# http://goo.gl/aeWc10

from django.conf import settings
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.tokens import default_token_generator
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, redirect
from django.template import RequestContext
from django.template.response import TemplateResponse
from django.urls import reverse
from django.utils.translation import ugettext as _ug
from django.views.decorators.csrf import csrf_protect

from . import messages as msgs
from .authenticationForms import PasswordResetForm


# Create your views here.
def LoginView(request):
    if request.user.is_authenticated():
        return redirect(request.get_host())

    else:
        email = password = ''
        next = request.GET.get('next', '')

        if request.method == 'POST':
            email = request.POST.get('email', '')
            password = request.POST.get('password', '')
            remember = request.POST.get('remember', None)
            user = authenticate(username=email, password=password)
            if user is not None:
                if user.is_active:
                    login(request, user)
                    if remember is not None:
                        request.session.set_expiry(72000)
                    else:
                        request.session.set_expiry(0)
                    msgs.generate_msg(
                        request, msgs.GREEN, " ",
                        _ug("Bienvenido de regreso!"))

                    if next == '' or next is None:
                        return redirect(request.get_host())
                    else:
                        return redirect(next)
                else:
                    msgs.generate_msg(
                        request,
                        msgs.ERROR,
                        msgs.errors_list['title']['400'],
                        msgs.errors_list['body']['bad_login']
                    )
            else:
                msgs.generate_msg(request, msgs.RED,
                                  msgs.errors_list['title']['400'],
                                  msgs.errors_list['body']['bad_login'])

        return render_to_response(
            'login.jinja',
            {'next': next, },
            context_instance=RequestContext(request),
        )


def LogoutView(request):
    logout(request)
    return redirect('%s' % (settings.LOGIN_URL))


@csrf_protect
def password_reset(request,
                   email_template_id,
                   template_name='registration/password_reset_form.html',
                   password_reset_form=PasswordResetForm,
                   token_generator=default_token_generator,
                   post_reset_redirect=None,
                   from_email=None,
                   extra_context=None,
                   html_email_template_name=None,
                   extra_email_context=None):
    if post_reset_redirect is None:
        post_reset_redirect = reverse('password_reset_done')
    else:
        post_reset_redirect = resolve_url(post_reset_redirect)
    if request.method == "POST":
        form = password_reset_form(request.POST)
        if form.is_valid():
            opts = {
                'use_https': request.is_secure(),
                'token_generator': token_generator,
                'from_email': from_email,
                'email_template_id': email_template_id,
                'request': request,
                'html_email_template_name': html_email_template_name,
                'extra_email_context': extra_email_context,
            }
            form.save(**opts)
            return HttpResponseRedirect(post_reset_redirect)
    else:
        form = password_reset_form()
    context = {
        'form': form,
        'title': _ug('Password reset'),
    }
    if extra_context is not None:
        context.update(extra_context)

    return TemplateResponse(request, template_name, context)



from graphene import relay, ObjectType

class CountableConnectionBase(relay.Connection):
    total_count = graphene.Int()
    
    class Meta:
        abstract = True

    def resolve_total_count(self, info, **kwargs):
        return self.iterable.count()


# Generated by Django 2.2.6 on 2019-10-17 19:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('configuration', '0002_auto_20191017_1646'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='organizationinfosettings',
            options={'verbose_name': 'Organization', 'verbose_name_plural': 'Organization'},
        ),
    ]



from time import time as timer
import logging

logger = logging.getLogger(__name__)

def log_db_queries(f):
    from django.db import connection
    from functools import reduce

    def new_f(*args, **kwargs):
        res = f(*args, **kwargs)
        print("-" * 80)
        print("db queries log for %s:\n" % (f.__name__))
        print("TOTAL COUNT: %s" % len(connection.queries))
        print("TOTAL TIME:  %s\n" % reduce(lambda x, y: x + float(y["time"]), connection.queries, 0.0))
        for q in connection.queries:
            print("%s:  %s\n" % (q["time"], q["sql"]))
        print("-" * 80)
        return res
    return new_f


def GrapheneTimingMiddleware(next, root, info, **args):
    start = timer()
    return_value = next(root, info, **args)
    duration = timer() - start
    logger.debug("{parent_type}.{field_name}: {duration} ms".format(
        parent_type=root._meta.name if root and hasattr(root, '_meta') else '',
        field_name=info.field_name,
        duration=round(duration * 1000, 2)
    ))
    return return_value



from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.db.models import Q


class EmailOrUsernameModelBackend(ModelBackend):
    """
    Authentication backend which allows users to authenticate using either their
    username or email address

    Source: https://stackoverflow.com/a/35836674/59984
    """

    def authenticate(self, request, username=None, password=None, **kwargs):
        # n.b. Django <2.1 does not pass the `request`

        user_model = get_user_model()

        if username is None:
            username = kwargs.get(user_model.USERNAME_FIELD)

        # The `username` field is allows to contain `@` characters so
        # technically a given email address could be present in either field,
        # possibly even for different users, so we'll query for all matching
        # records and test each one.
        users = user_model._default_manager.filter(
            Q(**{user_model.USERNAME_FIELD: username}
              ) | Q(email__iexact=username)
        )

        # Test whether any matched user has the provided password:
        for user in users:
            if user.check_password(password):
                return user
        if not users:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (see
            # https://code.djangoproject.com/ticket/20760)
            user_model().set_password(password)



# Obtained from: https://www.caktusgroup.com/blog/2014/11/10/Using-Amazon-S3-to-store-your-Django-sites-static-and-media-files/
# custom_storages.py
from django.conf import settings
from storages.backends.s3boto import S3BotoStorage


class StaticStorage(S3BotoStorage):
    location = settings.STATICFILES_LOCATION


class MediaStorage(S3BotoStorage):
    location = settings.MEDIAFILES_LOCATION
